"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const state = require("@codemirror/state");
const view = require("@codemirror/view");
const utils = require("@codemirror-toolkit/utils");
const ExtensionsCompartment = /* @__PURE__ */ new state.Compartment();
function isExtensionList(extension) {
  return Array.isArray(extension);
}
function updateExtensions(view2, extensions) {
  view2.dispatch({
    effects: ExtensionsCompartment.reconfigure(extensions)
  });
}
const ExtensionSetFacet = /* @__PURE__ */ state.Facet.define({
  combine(values) {
    return values.length ? values[0] : null;
  },
  static: true
});
function extensionManager(...initialExtensions) {
  return [
    ExtensionsCompartment.of(initialExtensions),
    ExtensionSetFacet.of(new Set(initialExtensions))
  ];
}
function wrapExtension(extension) {
  return { extension };
}
function getExtensionSet(state2) {
  const extensionSet = state2.facet(ExtensionSetFacet);
  if (!extensionSet) {
    throw new Error("extensionManager extension is not enabled");
  }
  return extensionSet;
}
function flattenExtensions(extensions, depth) {
  if (depth <= 0) {
    return extensions;
  }
  return extensions.flatMap((extension) => {
    if (!isExtensionList(extension)) {
      return extension;
    }
    return flattenExtensions(extension, depth - 1);
  });
}
function addExtension(view2, extension, { flattenDepth = 0 } = {}) {
  const extensionSet = getExtensionSet(view2.state);
  const flattenedExtensions = flattenExtensions([extension], flattenDepth);
  let didAdd = false;
  for (const ext of flattenedExtensions) {
    if (!extensionSet.has(ext)) {
      extensionSet.add(ext);
      didAdd = true;
    }
  }
  if (didAdd) {
    updateExtensions(view2, [...extensionSet]);
    return () => {
      removeExtension(view2, flattenedExtensions, { flattenDepth: 1 });
    };
  }
}
function removeExtension(view2, extension, { flattenDepth = 0 } = {}) {
  const extensionSet = getExtensionSet(view2.state);
  const flattenedExtensions = flattenExtensions([extension], flattenDepth);
  let didRemove = false;
  for (const ext of flattenedExtensions) {
    if (extensionSet.delete(ext)) {
      didRemove = true;
    }
  }
  if (didRemove) {
    updateExtensions(view2, [...extensionSet]);
  }
}
function clearExtensions(view2) {
  const extensionSet = getExtensionSet(view2.state);
  if (extensionSet.size) {
    extensionSet.clear();
    updateExtensions(view2, []);
  }
}
function hasExtension(view2, extension) {
  const extensionSet = getExtensionSet(view2.state);
  return extensionSet.has(extension);
}
const UpdateListenerEffect = /* @__PURE__ */ state.StateEffect.define();
const updateListenerField = /* @__PURE__ */ state.StateField.define({
  create() {
    return /* @__PURE__ */ new Set();
  },
  update(listeners, transaction) {
    transaction.effects.filter(utils.isEffectOfType(UpdateListenerEffect)).forEach(
      utils.mapEffectValue((action) => {
        var _a, _b;
        (_a = action.add) == null ? void 0 : _a.forEach((listener) => listeners.add(listener));
        (_b = action.remove) == null ? void 0 : _b.forEach((listener) => listeners.delete(listener));
      })
    );
    return listeners;
  },
  provide(thisField) {
    return view.EditorView.updateListener.from(
      thisField,
      (listeners) => function aggregatedListener(update) {
        listeners.forEach((listener) => listener(update));
      }
    );
  }
});
function updateListener(...initialListeners) {
  return updateListenerField.init(() => new Set(initialListeners));
}
function assertUpdateListenerField(view2) {
  if (!view2.state.field(
    updateListenerField,
    /* require: */
    false
  )) {
    throw new Error("updateListener extension is not enabled");
  }
}
function addUpdateListener(view2, listener) {
  assertUpdateListenerField(view2);
  view2.dispatch({
    effects: UpdateListenerEffect.of({ add: [listener] })
  });
  return () => {
    removeUpdateListener(view2, listener);
  };
}
function removeUpdateListener(view2, listener) {
  assertUpdateListenerField(view2);
  view2.dispatch({
    effects: UpdateListenerEffect.of({ remove: [listener] })
  });
}
exports.UpdateListenerEffect = UpdateListenerEffect;
exports.addExtension = addExtension;
exports.addUpdateListener = addUpdateListener;
exports.clearExtensions = clearExtensions;
exports.extensionManager = extensionManager;
exports.hasExtension = hasExtension;
exports.removeExtension = removeExtension;
exports.removeUpdateListener = removeUpdateListener;
exports.updateListener = updateListener;
exports.wrapExtension = wrapExtension;
//# sourceMappingURL=index.cjs.map
