import { Compartment, Facet, StateEffect, StateField } from "@codemirror/state";
import { EditorView } from "@codemirror/view";
import { isEffectOfType, mapEffectValue } from "@codemirror-toolkit/utils";
const ExtensionsCompartment = /* @__PURE__ */ new Compartment();
function isExtensionList(extension) {
  return Array.isArray(extension);
}
function updateExtensions(view, extensions) {
  view.dispatch({
    effects: ExtensionsCompartment.reconfigure(extensions)
  });
}
const ExtensionSetFacet = /* @__PURE__ */ Facet.define({
  combine(values) {
    return values.length ? values[0] : null;
  },
  static: true
});
function extensionManager(...initialExtensions) {
  return [
    ExtensionsCompartment.of(initialExtensions),
    ExtensionSetFacet.of(new Set(initialExtensions))
  ];
}
function wrapExtension(extension) {
  return { extension };
}
function getExtensionSet(state) {
  const extensionSet = state.facet(ExtensionSetFacet);
  if (!extensionSet) {
    throw new Error("extensionManager extension is not enabled");
  }
  return extensionSet;
}
function flattenExtensions(extensions, depth) {
  if (depth <= 0) {
    return extensions;
  }
  return extensions.flatMap((extension) => {
    if (!isExtensionList(extension)) {
      return extension;
    }
    return flattenExtensions(extension, depth - 1);
  });
}
function addExtension(view, extension, { flattenDepth = 0 } = {}) {
  const extensionSet = getExtensionSet(view.state);
  const flattenedExtensions = flattenExtensions([extension], flattenDepth);
  let didAdd = false;
  for (const ext of flattenedExtensions) {
    if (!extensionSet.has(ext)) {
      extensionSet.add(ext);
      didAdd = true;
    }
  }
  if (didAdd) {
    updateExtensions(view, [...extensionSet]);
    return () => {
      removeExtension(view, flattenedExtensions, { flattenDepth: 1 });
    };
  }
}
function removeExtension(view, extension, { flattenDepth = 0 } = {}) {
  const extensionSet = getExtensionSet(view.state);
  const flattenedExtensions = flattenExtensions([extension], flattenDepth);
  let didRemove = false;
  for (const ext of flattenedExtensions) {
    if (extensionSet.delete(ext)) {
      didRemove = true;
    }
  }
  if (didRemove) {
    updateExtensions(view, [...extensionSet]);
  }
}
function clearExtensions(view) {
  const extensionSet = getExtensionSet(view.state);
  if (extensionSet.size) {
    extensionSet.clear();
    updateExtensions(view, []);
  }
}
function hasExtension(view, extension) {
  const extensionSet = getExtensionSet(view.state);
  return extensionSet.has(extension);
}
const UpdateListenerEffect = /* @__PURE__ */ StateEffect.define();
const updateListenerField = /* @__PURE__ */ StateField.define({
  create() {
    return /* @__PURE__ */ new Set();
  },
  update(listeners, transaction) {
    transaction.effects.filter(isEffectOfType(UpdateListenerEffect)).forEach(
      mapEffectValue((action) => {
        var _a, _b;
        (_a = action.add) == null ? void 0 : _a.forEach((listener) => listeners.add(listener));
        (_b = action.remove) == null ? void 0 : _b.forEach((listener) => listeners.delete(listener));
      })
    );
    return listeners;
  },
  provide(thisField) {
    return EditorView.updateListener.from(
      thisField,
      (listeners) => function aggregatedListener(update) {
        listeners.forEach((listener) => listener(update));
      }
    );
  }
});
function updateListener(...initialListeners) {
  return updateListenerField.init(() => new Set(initialListeners));
}
function assertUpdateListenerField(view) {
  if (!view.state.field(
    updateListenerField,
    /* require: */
    false
  )) {
    throw new Error("updateListener extension is not enabled");
  }
}
function addUpdateListener(view, listener) {
  assertUpdateListenerField(view);
  view.dispatch({
    effects: UpdateListenerEffect.of({ add: [listener] })
  });
  return () => {
    removeUpdateListener(view, listener);
  };
}
function removeUpdateListener(view, listener) {
  assertUpdateListenerField(view);
  view.dispatch({
    effects: UpdateListenerEffect.of({ remove: [listener] })
  });
}
export {
  UpdateListenerEffect,
  addExtension,
  addUpdateListener,
  clearExtensions,
  extensionManager,
  hasExtension,
  removeExtension,
  removeUpdateListener,
  updateListener,
  wrapExtension
};
//# sourceMappingURL=index.js.map
